# servicelock.py - Provides classes to maintain a lock file.
#
# Copyright (C) 2009 Fabian Knittel <fabian.knittel@avona.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import os
import fcntl
import errno

LOCK_DIR = '/var/run'

class ServiceAlreadyRunning(Exception):
    pass

class ServiceRunLock(object):
    """A ServiceRunLock object aquires a lock file, typically in /var/run, for a
    specific service name. By holding such an object, you avoid running more
    than one instance of a service at a time."""

    def __init__(self, service_name):
        self._service_name = service_name
        self._fd = None

    def __del__(self):
        if self._fd is not None:
            self.release()

    @property
    def _lock_fn(self):
        return os.path.join(LOCK_DIR, self._service_name)

    def acquire(self):
        """Acquire the service lock."""
        # Attempt to open (and, if necessary, create) the lock file.
        fd = os.open(self._lock_fn, os.O_WRONLY | os.O_CREAT, 0666)
        # Attempt to lock the file.
        try:
            fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
        except EnvironmentError, err:
            os.close(fd)
            if err.errno == errno.EACCES or err.errno == errno.EAGAIN:
                raise ServiceAlreadyRunning(self._service_name)
            else:
                raise
        # Write our PID to the file.
        os.write(fd, "%d\n" % os.getpid())
        # Done.
        self._fd = fd

    def release(self):
        """Release the service lock.  This is done automatically on destruction
        of the object."""
        try:
            os.remove(self._lock_fn)
        except EnvironmentError, err:
            print err
            if err.errno != errno.ENOENT:
                raise
        os.close(self._fd)
        self._fd = None


